<?php

namespace AOP\App\Admin;

use AOP\App\Validation;
use AOP\Lib\Illuminate\Support\Collection;

class Notice extends Validation
{
    private static $noticeError        = 'notice notice-error';
    private static $noticeSuccesClass  = 'aop-notice aop-notice--succes';
    private static $delayClass         = ' has-delay';
    private static $noticeId           = 'aop-notice';
    private static $noticeMessageClass = 'aop-notice__message';

    public function show()
    {
        if ($this->notCorrectMenuTitle()) {
            return $this->errorNotCorrectMenuTitle($this->notCorrectMenuTitle());
        }

        if ($this->notCorrectMenuPosition()) {
            return $this->errorNotCorrectMenuPosition($this->notCorrectMenuPosition());
        }

        if (!$this->pagesFound()) {
            return $this->errorNoPagesFound($this->pagesFound());
        }

        if ($this->duplicatedFieldNames()) {
            return $this->errorDuplicatedFieldNames($this->duplicatedFieldNames());
        }

        if ($this->duplicatedOptionNames()) {
            return $this->errorDuplicatedOptionNames($this->duplicatedOptionNames());
        }

        if ($this->notCorrectFieldNames()) {
            return $this->errorNotCorrectFieldNames($this->notCorrectFieldNames());
        }
    }

    public static function succes($message = '', $delay = false)
    {
        printf(
            '<div class="aop-notice-wrapper"><div id="%s" class="%s%s"><p class="%s">%s</p></div></div>',
            self::$noticeId,
            self::$noticeSuccesClass,
            $delay === true ? self::$delayClass : '',
            self::$noticeMessageClass,
            htmlspecialchars($message)
        );

        wp_add_inline_script(
            'aop-app-js',
            'var aopNotice=document.getElementById("aop-notice");setTimeout(function(){aopNotice.classList.add("aop-notice--remove")},3e3),setTimeout(function(){aopNotice.outerHTML=""},5e3);',
            'after'
        );
    }

    public function errorDuplicatedFieldNames(array $names = [])
    {
        $names = Collection::make($names);

        if ($names->isEmpty()) {
            return;
        }

        printf('<div class="%s %s" %s>', self::$noticeError, self::$dismissible, self::$style);
        $names->map(function ($v, $key) {
            printf('<p>Field name <b>%s</b> is already in use on this page.</p>', $key);
        });
        print('</div>');
    }

    public function errorDuplicatedOptionNames(array $names = [])
    {
        $names = Collection::make($names);

        if ($names->isEmpty()) {
            return;
        }

        printf('<div class="%s %s" %s>', self::$noticeError, self::$dismissible, self::$style);
        $names->map(function ($name) {
            printf('<p>Field name <b>%s</b> is already in use as option name.</p>', $name);
        });
        print('</div>');
    }

    public function errorNotCorrectFieldNames(array $names = [])
    {
        $names = Collection::make($names);

        if ($names->isEmpty()) {
            return;
        }

        printf('<div class="%s %s" %s>', self::$noticeError, self::$dismissible, self::$style);
        $names->map(function ($name) {
            if ($name) {
                printf('<p>Field name <b>%s</b> is not allowed. Only letters, numbers and underscores are valid caracters.</p>', htmlspecialchars($name));
                print('<p>Use underscores instead of white spaces.</p>');
            } else {
                print('<p>Field name is not allowed to be empty.</p>');
            }
        });
        print('</div>');
    }

    public function errorNotCorrectMenuPosition($notCorrectNumber)
    {
        if ($notCorrectNumber === 'no request' || !$notCorrectNumber) {
            return;
        }

        printf('<div class="%s %s" %s>', self::$noticeError, self::$dismissible, self::$style);

        if ($notCorrectNumber === 'empty') {
            print('<p><b>Position</b> can not be empty.');
        }

        if ($notCorrectNumber !== 'empty') {
            printf('<p><b>Position</b> has not a correct number. Your input: <b>%s</b></p>', $notCorrectNumber);
        }

        print('</div>');
    }

    public function errorNotCorrectMenuTitle($notCorrectTitle)
    {
        if ($notCorrectTitle === 'no request' || !$notCorrectTitle) {
            return;
        }

        printf('<div class="%s %s" %s>', self::$noticeError, self::$dismissible, self::$style);

        if ($notCorrectTitle === 'empty') {
            print('<p><b>Menutitle</b> can not be empty.');
        }

        if ($notCorrectTitle !== 'empty') {
            // $notCorrectTitle = preg_replace('/\\\/', '', $notCorrectTitle);
            printf('<p><b>Menutitle</b> has some not valid characters: <b>%s</b></p>', $notCorrectTitle);
        }

        print('</div>');
    }

    public function errorNoPagesFound($boolean)
    {
        if (!$boolean) {
            return;
        }

        printf('<div class="%s %s" %s><p>There are no pages found.</p></div>', self::$noticeError, self::$dismissible, self::$style);
    }
}
